/******************************************************************************
* Header:
*	Trie Class Interface
*
* Confidence:
*	Customer 3/3
*
* Description:
*	This is a simple string to object mapping.
*
* Authors:
*	John W. M. Stevens (jstevens@fc.hp.com)
*
* Copyright:
*	(C) Copyright
*	1999-2007
*	John W. M. Stevens
******************************************************************************/

#if	!defined(TRIE_H)
#define	TRIE_H	1

/*	Defines for easy re-targeting.	*/

/*	Python defines.	*/
#define	GenObject	PyObject	*
#define	GenNULL		((GenObject) 0)
#define	GenList		PyListObject *

#define	Attach(a)	\
	Py_XINCREF( a )
#define	Dispose(a)	\
	Py_XDECREF( a )
#define	ListAppend(a, b)	\
	PyList_Append((GenObject) (a), (b))
#define	MakeStringObjectFromCString( a )	\
	PyString_FromString( strdup(a) )
#define	PerformObject(a, b, c)									\
{																\
	PyObject	*args;											\
	PyObject	*ret;											\
																\
	args = Py_BuildValue("(sO)", (b), (c));						\
	if ((ret = PyEval_CallObject((a), args)) != NULL)			\
		Py_DECREF( args );										\
	Py_DECREF( ret );											\
}
#define	PrintObject(a, b, c)	\
	PyObject_Print((a), (b), (c))

typedef	struct	{
	unsigned	char			c;
	struct		trie_node_st	*child;
	unsigned					subCount;
	GenObject					data;
} TrieElement;

typedef	struct	trie_node_st	{
	unsigned	used;
	TrieElement	els[1];
} TrieNode;

/*	Trie Class definition.	*/
typedef	struct	{
    PyObject_HEAD

	/*	Root of trie, and a count of keys.	*/
	TrieNode	*root;
	unsigned	total;
} TrieObject;

/*	Operation codes.	*/
typedef	enum	{
	TRIE_NOOP,
	TRIE_ASS_ITEM,
	TRIE_ASS_SLICE,
	TRIE_DATA,
	TRIE_DEBUG,
	TRIE_FREE,
	TRIE_INTERNAL,
	TRIE_ITEM,
	TRIE_KEYS,
	TRIE_PERFORM,
	TRIE_PRINT,
	TRIE_SLICE
} TrieOps;

typedef	void	(*DoFunc)(
					char		*key,
					PyObject	*data,
					PyObject	*other);

/*	Traversal control block.	*/
typedef	struct	{
	int			operation;
	char		*bfr;
	int			len;
	int			count;
	PyObject	*callObject;
	FILE		*fp;
	GenObject	list;
	GenObject	data;
	DoFunc		doFunc;
} Traversal;

/*	Traversal control block.	*/
typedef	struct	{
	int			operation;
	char		*bfr;
	int			len;
	int			lo;
	int			hi;
	int			count;
	GenObject	list;
} SeqTraversal;

#endif
